using System;
using System.IO;
using System.Text.Encodings.Web;
using System.Text.Unicode;
using FAuth.DataBase;
using FAuth.Extensions;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Caching.Redis;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.OpenApi.Models;

namespace FAuth
{
    /// <summary>
    /// Ӧ
    /// </summary>
    public class Startup
    {
        public Startup(IConfiguration configuration) {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        /// <summary>
        /// עᵽ
        /// </summary>
        /// <param name="services"></param>
        public void ConfigureServices(IServiceCollection services) {
            //עJsonл
            services.AddSingleton<IJsonProvider,MsJsonProvider>();
            //עݿ
            services.AddDbContext<FAuthDb>();
            //עRedis
            var rop = this.Configuration.GetSection("Redis").Get<RedisCacheOptions>();
            services.AddDistributedRedisCache(op => {
                op.InstanceName = rop.InstanceName;
                op.Configuration = rop.Configuration;
            });
            services.AddTransient<ICacheProvider,RedisCacheProvider>();
            //עMVC
            services.AddControllersWithViews()
                .AddJsonOptions(option => {
                    option.JsonSerializerOptions.Encoder = JavaScriptEncoder.Create(UnicodeRanges.All);
                    option.JsonSerializerOptions.PropertyNamingPolicy = null;
                });

            //עSwagger
            services.AddSwaggerGen(c => {
                c.SwaggerDoc("V1",new OpenApiInfo {
                    Title = "ӿĵ",
                    Version = "1.0",
                    Description = "",
                });
                var xmlPath = Path.Combine(AppContext.BaseDirectory,typeof(Program).Assembly.GetName().Name + ".xml");
                c.IncludeXmlComments(xmlPath,true);
                c.DocumentFilter<SwaggerXmlEnumFilter>(xmlPath);
            });
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app,IWebHostEnvironment env) {
            if(env.IsDevelopment()) {
                app.UseDeveloperExceptionPage();
            } else {
                app.UseExceptionHandler("/Home/Error");
                // The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
                app.UseHsts();
            }
            app.UseHttpsRedirection();
            app.UseStaticFiles();

            app.UseRouting();

            app.UseSwagger();
            app.UseSwaggerUI(c => {
                c.SwaggerEndpoint("/swagger/V1/swagger.json","ӿĵ");
                c.RoutePrefix = "";
            });

            app.UseAuthorization();

            app.UseEndpoints(endpoints => {
                endpoints.MapControllerRoute(
                    name: "default",
                    pattern: "{controller=Home}/{action=Index}/{id?}");
            });
        }
    }
}
